@@ -56,6 +56,7 @@
 import weakref
 from operator import attrgetter
 import sqlalchemy, sqlalchemy.orm
+from sqlalchemy.ext.declarative import DeclarativeMeta as BaseDeclarativeMeta
 
 
 __all__ = [
@@ -63,7 +64,7 @@ __all__ = [
     'PathOverflowError', 'TooManyChildrenError', 'PathTooDeepError'
 ]
 
-__version__ = (0, 4, 1)
+__version__ = (0, 5)
 __doc__ %= {'version': '.'.join(map(str, __version__))}
 
 
@@ -91,7 +92,7 @@ def inc_path(path, steplen):
     :param steplen:
         `int`, the number of maximum characters to carry overflow.
     :returns:
-        new path wich is greater than `path` by one.
+        new path which is greater than `path` by one.
     :raises PathOverflowError:
         when incrementation of `path` cause to carry overflow by number
         of characters greater than `steplen`.
@@ -126,47 +127,67 @@ class MPOptions(object):
 
     :parameters: see :class:`MPManager`.
     """
-    def __init__(self, table, pk_field, parent_id_field,
-                 path_field_name='mp_path',
-                 depth_field_name='mp_depth',
-                 tree_id_field_name='mp_tree_id',
+    def __init__(self,
+                 table,
+                 parent_id_field=None,
+                 path_field='mp_path',
+                 depth_field='mp_depth',
+                 tree_id_field='mp_tree_id',
                  steplen=3):
+
         self.table = table
 
-        def _check_field(field):
-            """
-            Check field argument (`pk_field`, `parent_id_field`) and
-            convert it from field name to `Column` object if needed.
-            """
-            if isinstance(field, basestring):
-                return table.c[field]
-            else:
-                assert field.table is table
-                return field
-
-        self.pk_field = _check_field(pk_field)
-        self.parent_id_field = _check_field(parent_id_field)
         self.steplen = steplen
 
         self.max_children = len(ALPHABET) ** steplen
         self.max_depth = (PATH_FIELD_LENGTH / steplen) + 1
 
-        self.path_field = sqlalchemy.Column(
-            path_field_name, PathField(PATH_FIELD_LENGTH), nullable=False
-        )
-        self.depth_field = sqlalchemy.Column(
-            depth_field_name, DepthField(), nullable=False
-        )
-        self.tree_id_field = sqlalchemy.Column(
-            tree_id_field_name, TreeIdField(), nullable=False
-        )
+        assert len(table.primary_key.columns) == 1, \
+               "Composite primary keys not supported"
+        [self.pk_field] = table.primary_key.columns
+
+        if parent_id_field is None:
+            self.parent_id_field = table.join(table).onclause.right
+        elif isinstance(parent_id_field, basestring):
+            self.parent_id_field = table.columns[parent_id_field]
+        else:
+            assert isinstance(parent_id_field, sqlalchemy.Column)
+            assert parent_id_field.table is table
+            self.parent_id_field = parent_id_field
+
+        def _check_field(name, field, type_):
+            # Check field argument (one of `path_field`, `depth_field` and
+            # `tree_id_field`),convert it from field name to `Column` object
+            # if needed, create the column object if needed and check the
+            # existing `Column` object for sane.
+            assert field
+            if not isinstance(field, basestring):
+                assert isinstance(field, sqlalchemy.Column)
+                assert field.table is table
+            elif field in table.columns:
+                field = table.columns[field]
+            else:
+                field = sqlalchemy.Column(field, type_(), nullable=False)
+                table.append_column(field)
+                return field
+            assert isinstance(field.type, type_), \
+                   "The type of %s field should be %r" % (name, type_)
+            assert not field.nullable, \
+                   "The %s field should not be nullable" % name
+            return field
+
+        self.path_field = _check_field('path', path_field, PathField)
+        self.depth_field = _check_field('depth', depth_field, DepthField)
+        self.tree_id_field = _check_field('tree_id', tree_id_field, TreeIdField)
         self.fields = (self.path_field, self.depth_field, self.tree_id_field)
-        map(table.append_column, self.fields)
 
         self.indices = [
             sqlalchemy.Index(
-                '%s__%s' % (tree_id_field_name, path_field_name),
-                self.tree_id_field, self.path_field, unique=True
+                '__'.join((table.name, self.tree_id_field.name,
+                           self.path_field.name)),
+                self.tree_id_field,
+                self.path_field,
+                unique=True
             ),
         ]
         map(table.append_constraint, self.indices)
@@ -247,7 +268,7 @@ class _InsertionsParamsSelector(object):
                     path = inc_path(query['last_child_path'], steplen)
                 except PathOverflowError:
                     # transform exception `PathOverflowError`, raised by
-                    # `inc_path()` to more convinient `TooManyChildrenError`.
+                    # `inc_path()` to more convenient `TooManyChildrenError`.
                     raise TooManyChildrenError()
             if len(path) > PATH_FIELD_LENGTH:
                 raise PathTooDeepError()
@@ -267,7 +288,7 @@ class _InsertionsParamsSelector(object):
 
 
 class TreeIdField(sqlalchemy.types.TypeDecorator):
-    "Integer field representing node's tree identifier."
+    "Integer field subtype representing node's tree identifier."
     impl = sqlalchemy.Integer
     def process_bind_param(self, value, dialect):
         if not isinstance(value, _InsertionsParamsSelector):
@@ -275,7 +296,7 @@ class TreeIdField(sqlalchemy.types.TypeDecorator):
         return value.query_result['tree_id']
 
 class DepthField(sqlalchemy.types.TypeDecorator):
-    "Integer field representing node's depth level."
+    "Integer field subtype representing node's depth level."
     impl = sqlalchemy.Integer
     def process_bind_param(self, value, dialect):
         if not isinstance(value, _InsertionsParamsSelector):
@@ -283,8 +304,10 @@ class DepthField(sqlalchemy.types.TypeDecorator):
         return value.query_result['depth']
 
 class PathField(sqlalchemy.types.TypeDecorator):
-    "Varchar field representing node's path."
+    "Varchar field subtype representing node's path."
     impl = sqlalchemy.String
+    def __init__(self):
+        super(PathField, self).__init__(PATH_FIELD_LENGTH)
     def process_bind_param(self, value, dialect):
         if not isinstance(value, _InsertionsParamsSelector):
             return value
@@ -462,7 +485,7 @@ class MPClassManager(object):
 
         :param session: a sqlalchemy `Session` object to bind a query.
         :returns:
-            `Query` object whith all nodes of all trees sorted as usual
+            `Query` object with all nodes of all trees sorted as usual
             by `(tree_id, path)`.
         """
         query = sqlalchemy.orm.Query(self.node_class, session=session) \
@@ -684,12 +707,12 @@ class MPManager(object):
     """
     Descriptor for access class-level and instance-level API.
 
-    Usage is simple::
+    Basic usage is simple::
 
         class Node(object):
-            mp = sqlamp.MPManager(...)
+            mp = sqlamp.MPManager(node_table)
 
-    Now there are an ability to get instance manager or class manager
+    Now there is an ability to get instance manager or class manager
     via property `'mp'` depending on way to access it. `Node.mp` will
     return class manager :class:`MPClassManager` for class `Node` and
     `instance_node.mp` will return instance_node's
@@ -698,25 +721,38 @@ class MPManager(object):
 
     :param table:
         instance of `sqlalchemy.Table`. A table that will be mapped to
-        node class and will hold tree nodes in its rows.
-    :param pk_field:
-        the `table`'s primary key column, may be an instance
-        of `sqlalchemy.Column` or field name as string.
-    :param parent_id_field:
+        node class and will hold tree nodes in its rows. It is the only
+        one strictly required argument.
+
+    :param parent_id_field=None:
         a foreign key field that is reference to parent node's
-        primary key.
-    :param path_field_name='mp_path':
-        name for the path field.
-    :param depth_field_name='mp_depth':
-        name for the depth field.
-    :param tree_id_field_name='mp_tree_id':
-        name for the tree_id field.
+        primary key. If this parameter is omitted, it will be guessed
+        joining a `table` with itself and using the right part of join's
+        onclause as parent id field.
+
+    :param path_field='mp_path':
+        the hame for the path field or the field object itself. The field
+        will be created if the actual parameter value is a string and
+        there is no such column in the table `table`. If value provided
+        is an object column some sanity checks will be performed with
+        the column object: it should have `nullable=False` and have
+        :class:`PathField` type.
+
+    :param depth_field='mp_depth':
+        the same as for :attr:`path_field`, except that the type of this
+        column should be :class:`DepthField`.
+
+    :param tree_id_field='mp_tree_id':
+        the same as for :attr:`path_field`, except that the type of this
+        column should be :class:`TreeIdField`.
+
     :param steplen=3:
-        integer, the number of characters in each part of the path.
+        an integer, the number of characters in each part of the path.
         This value allows to fine-tune the limits for max tree depth
         (equal to `(255 / steplen) + 1`) and max children in each node
         (`36 ** steplen`). Default `3` value sets the following limits:
         max depth is `86` and max children number is `46656`.
+
     :param instance_manager_key='_mp_instance_manager':
         name for node instance's attribute to cache node's instance
         manager.
@@ -863,3 +899,51 @@ def tree_recursive_iterator(flat_tree, class_manager):
         flat_tree = flat_tree.order_by(class_manager)
     return _recursive_iterator(flat_tree, is_child)
 
+
+class DeclarativeMeta(BaseDeclarativeMeta):
+    """
+    Metaclass for declaratively defined node model classes.
+
+    .. versionadded:: 0.5
+
+    See :ref:`usage example <declarative>` above in Quickstart.
+
+    All options that accepts :class:`MPManager` can be provided
+    with declarative definition. To provide an option you can
+    simply assign value to class' property with name like
+    ``__mp_tree_id_field__`` (for ``tree_id_field`` parameter)
+    and so forth. See the complete list of options in :class:`MPManager`'s
+    constructor parameters. Note that you can use only string options
+    for field names, not the column objects.
+
+    A special class variable ``__mp_manager__`` should exist and hold
+    a string name which will be used as `MPManager` descriptor property.
+    """
+    def __init__(cls, name, bases, dct):
+        if not hasattr(cls, '__mp_manager__'):
+            super(DeclarativeMeta, cls).__init__(name, bases, dct)
+            return
+        mp_manager_name = cls.__mp_manager__
+        opts = {}
+        for opt, default in [
+                ('path_field', 'mp_path'),
+                ('depth_field', 'mp_depth'),
+                ('tree_id_field', 'mp_tree_id'),
+                ('steplen', 3),
+                ('instance_manager_key', '_mp_instance_manager')
+            ]:
+            opts[opt] = getattr(cls, '__mp_%s__' % opt, default)
+        for field, ftype in [
+                ('path_field', PathField),
+                ('depth_field', DepthField),
+                ('tree_id_field', TreeIdField)
+            ]:
+            assert isinstance(opts[field], basestring)
+            if not hasattr(cls, opts[field]):
+                dct[opts[field]] = sqlalchemy.Column(ftype(), nullable=False)
+        super(DeclarativeMeta, cls).__init__(name, bases, dct)
+        mp_manager = MPManager(cls.__table__, **opts)
+        setattr(cls, mp_manager_name, mp_manager)
+        mp_class_manager = getattr(cls, mp_manager_name)
+        cls.__mapper__.extension.append(mp_class_manager.mapper_extension)
+
